pub enum SoundBufferResourceLoadError {
UnsupportedFormat,
Io(FileError),
DataSourceError,
SoundError(SoundError),
}Expand description
An error that can occur during loading of sound buffer.
Variants
UnsupportedFormat
A format is not supported.
Io(FileError)
File load error.
DataSourceError
Errors involving the data source
This could be, e.g., wrong number of channels, or attempting to use a
DataSource::RawStreaming where it doesn’t make sense
SoundError(SoundError)
Underlying sound error