StreamingBuffer

Struct StreamingBuffer 

pub struct StreamingBuffer { /* private fields */ }
Streaming buffer for long sounds. Does not support random access.

Implementations

Source

impl StreamingBuffer

pub const GENERIC: &'static str = "generic"

pub const USE_COUNT: &'static str = "use_count"

Source

impl StreamingBuffer

pub const STREAM_SAMPLE_COUNT: usize = 44100

Defines amount of samples per channel which each streaming buffer will use for internal buffer.

pub fn new(source: DataSource) -> Result<Self, SoundError>

Creates new streaming buffer using given data source. May fail if data source has unsupported format or it has corrupted data. Length of internal generic buffer cannot be changed but can be fetched from StreamingBuffer::STREAM_SAMPLE_COUNT

Notes

This function will return Err if data source is Raw. It makes no sense to stream raw data which is already loaded into memory. Use Generic source instead!

Methods from Deref<Target = GenericBuffer>

pub const SAMPLES: &'static str = "samples"

pub const CHANNEL_COUNT: &'static str = "channel_count"

pub const SAMPLE_RATE: &'static str = "sample_rate"

pub const CHANNEL_DURATION_IN_SAMPLES: &'static str = "channel_duration_in_samples"

pub fn is_empty(&self) -> bool

Checks if buffer is empty or not.

pub fn samples(&self) -> &[f32]

Returns shared reference to an array with samples.

pub fn samples_mut(&mut self) -> &mut [f32]

Returns mutable reference to an array with samples that could be modified.

pub fn channel_count(&self) -> usize

Returns exact amount of channels in the buffer.

pub fn sample_rate(&self) -> usize

Returns sample rate of the buffer.

pub fn duration(&self) -> Duration

Returns exact time length of the buffer.

pub fn channel_duration_in_samples(&self) -> usize

Returns exact duration of each channel (in samples) of the buffer. The returned value represents the entire length of each channel in the buffer, even if it is streaming and its content is not yet fully decoded.