pub struct SerializationOptions {
pub skip_sources: bool,
pub skip_bus_graph: bool,
}Expand description
A set of flags, that can be used to define what should be skipped during the serialization of a sound context.
Fields
skip_sources: boolAll sources won’t be serialized, if set.
skip_bus_graph: boolBus graph won’t be serialized, if set.