pub struct GenericBuffer { /* private fields */ }Expand description
Generic sound buffer that contains decoded samples and allows random access.
Implementations
Sourceimpl GenericBuffer
impl GenericBuffer
pub const SAMPLES: &'static str = "samples"
pub const CHANNEL_COUNT: &'static str = "channel_count"
pub const SAMPLE_RATE: &'static str = "sample_rate"
pub const CHANNEL_DURATION_IN_SAMPLES: &'static str = "channel_duration_in_samples"
Sourceimpl GenericBuffer
impl GenericBuffer
Sourcepub fn new(source: DataSource) -> Result<Self, SoundBufferResourceLoadError>
pub fn new(source: DataSource) -> Result<Self, SoundBufferResourceLoadError>
Creates new generic buffer from specified data source. May fail if data source has unsupported format, corrupted, etc.
Notes
DataSource::RawStreaming is not supported with generic buffers, use streaming buffer
instead!
Data source with raw samples must have sample count multiple of channel count, otherwise this
function will return Err.
Sourcepub fn samples_mut(&mut self) -> &mut [f32]
pub fn samples_mut(&mut self) -> &mut [f32]
Returns mutable reference to an array with samples that could be modified.
Sourcepub fn channel_count(&self) -> usize
pub fn channel_count(&self) -> usize
Returns exact amount of channels in the buffer.
Sourcepub fn sample_rate(&self) -> usize
pub fn sample_rate(&self) -> usize
Returns sample rate of the buffer.
Sourcepub fn channel_duration_in_samples(&self) -> usize
pub fn channel_duration_in_samples(&self) -> usize
Returns exact duration of each channel (in samples) of the buffer. The returned value represents the entire length of each channel in the buffer, even if it is streaming and its content is not yet fully decoded.